import { Component, Prop, h, Watch } from '@stencil/core';
import { Parser } from '../../infrastructure/parser';
import { Renderer } from '../../node-renderers/renderer';
export class XmlViewerComponent {
  constructor() {
    this.useUnescapedNodeValue = true;
    this.nodeRenderer = new Renderer();
  }
  componentDidLoad() {
  }
  xmlPropChanged() {
    this.render();
  }
  useUnescapedNodeValuePropChanged() {
    this.render();
  }
  // for FireFox and maybe Edge.
  toArray(obj) {
    let arr = [];
    for (let i = 0; i < obj.length; i++) {
      arr.push(obj[i]);
    }
    return arr;
  }
  prepareXml() {
    let xdoc = Parser.Parse(this.xml.trim());
    return xdoc;
  }
  renderAttribute(attribute) {
    return (h("span", null,
      "\u00A0",
      h("span", { class: "attr" },
        attribute.name,
        "=\"",
        h("span", { class: "attr-value" }, attribute.value),
        "\"")));
  }
  renderNodeValue(nodeValue) {
    if (!nodeValue) {
      return null;
    }
    const val = this.useUnescapedNodeValue ? unescape(nodeValue.trim()) : nodeValue.trim();
    if (val.length === 0) {
      return null;
    }
    if (val.length > 150) {
      return (h("ul", null,
        h("li", null, val)));
    }
    return (h("span", null, val));
  }
  renderNode(node) {
    return this.nodeRenderer.renderNode(node);
  }
  // rendering node. This function calls to itself in recursion way in case of child nodes
  renderElement(element) {
    if (!element) {
      return null;
    }
    // element.attributes, childNodes and children has no iterator so I have to turn it into an array
    const attributes = this.toArray(element.attributes);
    const childNodes = this.toArray(element.childNodes);
    const children = this.toArray(element.children);
    let nodeValue = element.firstChild ? element.firstChild.nodeValue : null;
    nodeValue = nodeValue ? nodeValue.trim() : null;
    return (h("ul", null,
      h("li", null,
        h("div", { class: "element" },
          "<",
          element.nodeName,
          attributes.map(a => this.renderAttribute(a)),
          ">"),
        h("div", { class: "element-content" }, this.renderNodeValue(nodeValue)),
        children.map(c => this.renderElement(c)),
        childNodes.map(c => this.renderNode(c)),
        h("div", { class: "element" },
          "</",
          element.nodeName,
          ">"))));
  }
  render() {
    if (this.xml === null || this.xml === undefined) {
      return null;
    }
    if (this.xml.length === 0) {
      return null;
    }
    let xdoc = null;
    try {
      xdoc = this.prepareXml();
    }
    catch (e) {
      console.error(e);
    }
    if (!xdoc || !xdoc.documentElement.childNodes) {
      return (h("code", null, this.xml));
    }
    return (h("p", null,
      h("code", null, this.renderElement(xdoc.documentElement))));
  }
  static get is() { return "xml-viewer-component"; }
  static get encapsulation() { return "shadow"; }
  static get originalStyleUrls() { return {
    "$": ["xml-viewer-component.css"]
  }; }
  static get styleUrls() { return {
    "$": ["xml-viewer-component.css"]
  }; }
  static get properties() { return {
    "xml": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      },
      "attribute": "xml",
      "reflect": false
    },
    "useUnescapedNodeValue": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      },
      "attribute": "use-unescaped-node-value",
      "reflect": false,
      "defaultValue": "true"
    }
  }; }
  static get watchers() { return [{
      "propName": "xml",
      "methodName": "xmlPropChanged"
    }, {
      "propName": "useUnescapedNodeValue",
      "methodName": "useUnescapedNodeValuePropChanged"
    }]; }
}
